﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using Android;
using Android.App;
using Android.Content;
using Android.Content.PM;
using Android.OS;
using Android.Runtime;
using Android.Support.V4.App;
using Android.Support.V4.Content;
using Android.Views;
using Android.Widget;
using Java.IO;
using Syncfusion.Pdf;
using Syncfusion.Pdf.Graphics;
using Syncfusion.SfChart.XForms;
using Syncfusion.SfChart.XForms.Droid;
using Xamarin.Forms;

namespace ChartToPdfExample.Droid
{
    class ExportChartToPdf : IChartDependencyService
    {
        public void SaveAsPDF(Stream chartStream, SfChart chart, PdfDocument document)
        {
            if (ContextCompat.CheckSelfPermission(Android.App.Application.Context, Manifest.Permission.WriteExternalStorage) != Permission.Granted)
            {
                ActivityCompat.RequestPermissions((Android.App.Activity)Forms.Context, new String[] { Manifest.Permission.WriteExternalStorage }, 1);
            }
            if (ContextCompat.CheckSelfPermission(Android.App.Application.Context, Manifest.Permission.ReadExternalStorage) != Permission.Granted)
            {
                ActivityCompat.RequestPermissions((Android.App.Activity)Forms.Context, new String[] { Manifest.Permission.ReadExternalStorage }, 1);
            }
            try
            {
                var page = document.Pages.Add();
                var graphics = page.Graphics;

                graphics.DrawImage(PdfImage.FromStream(chartStream), 0, 0, page.GetClientSize().Width,
                page.GetClientSize().Height);
                MemoryStream stream = new MemoryStream();
                document.Save(stream);
                document.Close(true);
                SavePDF("signal.pdf", stream, chart);
            }
            finally
            {
                chartStream.Flush();
                chartStream.Close();
                var nativeChart = SfChartRenderer.GetNativeObject(typeof(SfChart), chart);
                if (((Com.Syncfusion.Charts.SfChart)nativeChart).Parent != null)
                {
                    (((Com.Syncfusion.Charts.SfChart)nativeChart).Parent as ViewGroup).RemoveAllViews();
                }
                ((Com.Syncfusion.Charts.SfChart)nativeChart).DrawingCacheEnabled = false;
            }
        }
        private void SavePDF(string fileName, MemoryStream stream, SfChart chart)
        {
            var root = Android.OS.Environment.IsExternalStorageEmulated
                    ? Android.OS.Environment.ExternalStorageDirectory.ToString()
                    : System.Environment.GetFolderPath(System.Environment.SpecialFolder.MyDocuments);

            var myDir = new Java.IO.File(root + "/Chart");
            myDir.Mkdir();

            var file = new Java.IO.File(myDir, fileName);

            if (file.Exists()) file.Delete();

            try
            {
                var outs = new FileOutputStream(file);
                outs.Write(stream.ToArray());

                outs.Flush();
                outs.Close();

                var nativeChart = SfChartRenderer.GetNativeObject(typeof(SfChart), chart);

                ((Com.Syncfusion.Charts.SfChart)nativeChart).DrawingCacheEnabled = false;
            }
            catch (Exception e)
            {
                string msg = e.ToString();
            }
            if (!file.Exists()) return;
            var path = Android.Net.Uri.FromFile(file);
            var extension = Android.Webkit.MimeTypeMap.GetFileExtensionFromUrl(Android.Net.Uri.FromFile(file).ToString());
            var mimeType = Android.Webkit.MimeTypeMap.Singleton.GetMimeTypeFromExtension(extension);
            var intent = new Intent(Intent.ActionView);
            intent.SetDataAndType(path, mimeType);
            Forms.Context.StartActivity(Intent.CreateChooser(intent, "Choose App"));
        }
    }
}
